// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';

part 'account.g.dart';

/// An user account.
abstract class Account extends Object
    with EntityMixin
    implements Built<Account, AccountBuilder> {
  static Serializer<Account> get serializer => _$accountSerializer;

  factory Account.build([void Function(AccountBuilder b) updates]) = _$Account;

  Account._();

  /// Preferred currency.
  @nullable
  String get currencyCode;

  /// Email address.
  @nullable
  EmailAddress get emailAddress;

  /// Preferred locale.
  @nullable
  String get localeName;

  /// System-specific user name. Only some characters may be allowed.
  /// May be undefined.
  @nullable
  String get name;

  /// Organization who uses this account.
  @nullable
  Organization get organization;

  /// Person who uses this account.
  @nullable
  Person get person;

  /// Primary phone number.
  @nullable
  PhoneNumber get phoneNumber;

  /// Place. May affect data residency.
  @nullable
  Place get place;

  /// One-line summary such as real name.
  @nullable
  String get summary;

  /// Account type.
  @nullable
  String get type;

  /// Primary website URI.
  @nullable
  String get websiteUri;
}

/// Searches instances of [Account].
abstract class AccountQuery extends Object
    with EntityQueryMixin
    implements Built<AccountQuery, AccountQueryBuilder> {
  static Serializer<AccountQuery> get serializer => _$accountQuerySerializer;

  factory AccountQuery.build([void Function(AccountQueryBuilder b) updates]) =
      _$AccountQuery;

  AccountQuery._();

  /// Preferred currency.
  @nullable
  StringQuery get currencyCode;

  /// Primary email.
  @nullable
  EmailAddressQuery get emailAddress;

  /// Preferred locale.
  @nullable
  StringQuery get localeName;

  /// System-specific user name. Only some characters may be allowed.
  /// May be undefined.
  @nullable
  StringQuery get name;

  /// Organization associated with the account.
  @nullable
  OrganizationQuery get organization;

  /// Person associated with the account.
  @nullable
  PersonQuery get person;

  /// Phone number.
  @nullable
  PhoneNumberQuery get phoneNumber;

  /// Place. May affect data residency.
  @nullable
  PlaceQuery get place;

  /// Summary of the person.
  @nullable
  StringQuery get summary;

  /// Account type.
  @nullable
  String get type;
}
