// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/core.dart';

part 'account_provider.g.dart';

/// Describes an user account type.
abstract class AccountProvider extends Object
    with EntityMixin
    implements Built<AccountProvider, AccountProviderBuilder> {
  static Serializer<AccountProvider> get serializer =>
      _$accountProviderSerializer;

  factory AccountProvider.build(
      [void Function(AccountProviderBuilder b) updates]) = _$AccountProvider;

  AccountProvider._();

  /// Domain of the account provider (e.g. "linkedin.com".
  @nullable
  String get domain;

  /// Title of the account provider (e.g. "LinkedIn").
  @nullable
  String get title;
}

/// An user.
abstract class AccountProviderQuery extends Object
    with EntityQueryMixin
    implements Built<AccountProviderQuery, AccountProviderQueryBuilder> {
  static Serializer<AccountProviderQuery> get serializer =>
      _$accountProviderQuerySerializer;

  factory AccountProviderQuery(
          [void Function(AccountProviderQueryBuilder b) updates]) =
      _$AccountProviderQuery;

  AccountProviderQuery._();

  /// Domain of the account provider (e.g. "linkedin.com".
  @nullable
  StringQuery get domain;

  /// Title of the account provider (e.g. "LinkedIn").
  @nullable
  StringQuery get title;
}
