// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';

part 'organization.g.dart';

/// An organization.
abstract class Organization extends Object
    with EntityMixin
    implements Built<Organization, OrganizationBuilder> {
  static Serializer<Organization> get serializer => _$organizationSerializer;

  factory Organization.build([void Function(OrganizationBuilder b) updates]) =
      _$Organization;

  Organization._();

  /// Email address.
  @nullable
  EmailAddress get emailAddress;

  /// Legal name of the organization, usually with some suffix.
  @nullable
  String get legalName;

  /// Possibly localized name of the organization.
  @nullable
  Text get name;

  @nullable
  Organization get parentOrganization;

  /// Phone number.
  @nullable
  PhoneNumber get phoneNumber;

  /// Place.
  @nullable
  Place get place;

  /// VAT code used in some places such as the EU.
  @nullable
  String get vatCode;

  /// Website URI.
  @nullable
  Text get websiteUri;
}

/// Searches instances of [Organization].
abstract class OrganizationQuery extends Object
    with EntityQueryMixin
    implements Built<OrganizationQuery, OrganizationQueryBuilder> {
  static Serializer<OrganizationQuery> get serializer =>
      _$organizationQuerySerializer;

  factory OrganizationQuery.build(
          [void Function(OrganizationQueryBuilder b) updates]) =
      _$OrganizationQuery;

  OrganizationQuery._();

  @nullable
  EmailAddress get emailAddress;

  @nullable
  StringQuery get legalName;

  @nullable
  TextQuery get name;

  @nullable
  OrganizationQuery get parentOrganizationQuery;

  @nullable
  PhoneNumberQuery get phoneNumber;

  @nullable
  PlaceQuery get place;

  @nullable
  StringQuery get vatCode;

  @nullable
  StringQuery get websiteUri;
}
