// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';
import 'package:database/database.dart';

part 'person.g.dart';

/// A natural person.
abstract class Person extends Object
    with EntityMixin
    implements Built<Person, PersonBuilder> {
  static Serializer<Person> get serializer => _$personSerializer;

  factory Person.build([void Function(PersonBuilder b) updates]) = _$Person;

  Person._();

  @nullable
  Date get birthDate;

  @nullable
  String get gender;

  @nullable
  Place get homePlace;

  @nullable
  String get localeName;

  @nullable
  PersonName get name;
}

/// Searches instances of [Person].
abstract class PersonQuery extends Object
    with EntityQueryMixin
    implements Built<PersonQuery, PersonQueryBuilder> {
  static Serializer<PersonQuery> get serializer => _$personQuerySerializer;

  factory PersonQuery.build([void Function(PersonQueryBuilder b) updates]) =
      _$PersonQuery;

  PersonQuery._();

  @nullable
  StringQuery get birthDate;

  @nullable
  StringQuery get gender;

  @nullable
  PlaceQuery get homePlace;

  @nullable
  StringQuery get localeName;

  @nullable
  PersonNameQuery get name;

  @nullable
  Place get place;
}
