// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';

part 'person_name.g.dart';

/// Name of a [Person].
abstract class PersonName extends Object
    with EntityMixin
    implements Built<PersonName, PersonNameBuilder> {
  static Serializer<PersonName> get serializer => _$personNameSerializer;

  factory PersonName.build([void Function(PersonNameBuilder b) updates]) =
      _$PersonName;

  PersonName._();

  /// First name of the person.
  @nullable
  String get firstName;

  /// English honorific of the person (e.g. "Dr", "Mr", etc.)
  @nullable
  String get honorifics;

  /// Last name of the person
  @nullable
  String get lastName;

  /// Name in unspecified format.
  @nullable
  String get name;
}

/// Searches instances of [PersonName].
abstract class PersonNameQuery extends Object
    with EntityQueryMixin
    implements Built<PersonNameQuery, PersonNameQueryBuilder> {
  static Serializer<PersonNameQuery> get serializer =>
      _$personNameQuerySerializer;

  factory PersonNameQuery.build(
      [void Function(PersonNameQueryBuilder b) updates]) = _$PersonNameQuery;

  PersonNameQuery._();
}
