// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

part 'phone_number.g.dart';

/// A phone number.
class PhoneNumber {
  static const Serializer serializer = _PhoneNumberSerializer();

  final String value;

  const PhoneNumber(this.value);

  @override
  int get hashCode => value.hashCode;

  @override
  bool operator ==(other) => other is PhoneNumber && value == other.value;

  @override
  String toString() => value;
}

/// Searches instances of [PhoneNumber].
abstract class PhoneNumberQuery
    implements Built<PhoneNumberQuery, PhoneNumberQueryBuilder> {
  static Serializer<PhoneNumberQuery> get serializer =>
      _$phoneNumberQuerySerializer;

  factory PhoneNumberQuery.build(
      [void Function(PhoneNumberQueryBuilder b) updates]) = _$PhoneNumberQuery;

  PhoneNumberQuery._();

  @nullable
  String get value;
}

class _PhoneNumberSerializer implements PrimitiveSerializer<PhoneNumber> {
  const _PhoneNumberSerializer();

  @override
  Iterable<Type> get types => const [PhoneNumber];

  @override
  String get wireName => 'phone_number';

  @override
  PhoneNumber deserialize(Serializers serializers, Object serialized,
      {FullType specifiedType = FullType.unspecified}) {
    return PhoneNumber(serialized as String);
  }

  @override
  Object serialize(Serializers serializers, PhoneNumber object,
      {FullType specifiedType = FullType.unspecified}) {
    return object.value;
  }
}
