// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';

part 'product.g.dart';

/// A product or service.
abstract class Product extends Object
    with EntityMixin
    implements Built<Product, ProductBuilder> {
  static Serializer<Product> get serializer => _$productSerializer;

  factory Product({
    Meta meta,
    Text description,
    Place place,
    Money price,
    Text title,
    String websiteUri,
  }) = _$Product._;

  factory Product.build([void Function(ProductBuilder b) updates]) = _$Product;

  Product._();

  @nullable
  Text get description;

  @nullable
  Place get place;

  @nullable
  Money get price;

  @nullable
  Text get title;

  @nullable
  String get websiteUri;
}

/// Searches instances of [Product].
abstract class ProductQuery extends Object
    with EntityQueryMixin
    implements Built<ProductQuery, ProductQueryBuilder> {
  static Serializer<ProductQuery> get serializer => _$productQuerySerializer;

  factory ProductQuery({
    MetaQuery meta,
    TextQuery description,
    PlaceQuery place,
    MoneyQuery price,
    TextQuery summary,
    TextQuery title,
  }) = _$ProductQuery._;

  factory ProductQuery.build([void Function(ProductQueryBuilder b) updates]) =
      _$ProductQuery;

  ProductQuery._();

  BuiltList<String> get categories;

  @nullable
  TextQuery get description;

  @nullable
  PlaceQuery get place;

  @nullable
  MoneyQuery get price;

  @nullable
  TextQuery get summary;

  @nullable
  TextQuery get title;

  @nullable
  StringQuery get websiteUri;
}
