// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';

part 'shopping_cart.g.dart';

/// A list of products purchased in a transaction.
abstract class ShoppingCart extends Object
    with EntityMixin
    implements Built<ShoppingCart, ShoppingCartBuilder> {
  static Serializer<ShoppingCart> get serializer => _$shoppingCartSerializer;

  factory ShoppingCart.build([void Function(ShoppingCartBuilder b) updates]) =
      _$ShoppingCart;

  ShoppingCart._();

  @nullable
  Organization get billingOrganization;

  @nullable
  String get billingOrganizationVatNumber;

  @nullable
  Person get billingPerson;

  @nullable
  Place get billingPlace;

  @nullable
  Organization get recipientOrganization;

  @nullable
  Person get recipientPerson;

  @nullable
  Place get recipientPlace;

  @nullable
  String get title;

  @nullable
  Money get total;
}

/// Searches instances of [ShoppingCart].
abstract class ShoppingCartQuery extends Object
    with EntityQueryMixin
    implements Built<ShoppingCartQuery, ShoppingCartQueryBuilder> {
  static Serializer<ShoppingCartQuery> get serializer =>
      _$shoppingCartQuerySerializer;

  factory ShoppingCartQuery({
    MetaQuery meta,
    StringQuery title,
    MoneyQuery total,
  }) = _$ShoppingCartQuery._;

  factory ShoppingCartQuery.build(
          [void Function(ShoppingCartQueryBuilder b) updates]) =
      _$ShoppingCartQuery;

  ShoppingCartQuery._();

  @nullable
  StringQuery get title;

  @nullable
  MoneyQuery get total;
}
