// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/business.dart';
import 'package:dent/core.dart';
import 'package:dent/world.dart';

part 'shopping_cart_item.g.dart';

/// A product or service.
abstract class ShoppingCartItem extends Object
    with EntityMixin
    implements Built<ShoppingCartItem, ShoppingCartItemBuilder> {
  static Serializer<ShoppingCartItem> get serializer =>
      _$shoppingCartItemSerializer;

  factory ShoppingCartItem({
    Meta meta,
    Product product,
    int quantity,
  }) = _$ShoppingCartItem._;

  factory ShoppingCartItem.build(
      [void Function(ShoppingCartItemBuilder b) updates]) = _$ShoppingCartItem;

  ShoppingCartItem._();

  @nullable
  Product get product;

  @nullable
  int get quantity;

  @nullable
  Place get recipientAddress;

  @nullable
  Organization get recipientOrganization;

  @nullable
  Person get recipientPerson;
}

/// Searches instances of [ShoppingCartItem].
abstract class ShoppingCartItemQuery extends Object
    with EntityQueryMixin
    implements Built<ShoppingCartItemQuery, ShoppingCartItemQueryBuilder> {
  static Serializer<ShoppingCartItemQuery> get serializer =>
      _$shoppingCartItemQuerySerializer;

  factory ShoppingCartItemQuery({
    MetaQuery meta,
    ProductQuery product,
    IntQuery quantity,
  }) = _$ShoppingCartItemQuery._;

  factory ShoppingCartItemQuery.build(
          [void Function(ShoppingCartItemQueryBuilder b) updates]) =
      _$ShoppingCartItemQuery;

  ShoppingCartItemQuery._();

  @nullable
  ProductQuery get product;

  @nullable
  IntQuery get quantity;
}
