// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/core.dart';

part 'shopping_transaction.g.dart';

/// A product or service.
abstract class ShoppingTransaction extends Object
    with EntityMixin
    implements Built<ShoppingTransaction, ShoppingTransactionBuilder> {
  static Serializer<ShoppingTransaction> get serializer =>
      _$shoppingTransactionSerializer;

  factory ShoppingTransaction({
    Meta meta,
    Money total,
  }) = _$ShoppingTransaction._;

  factory ShoppingTransaction.build([
    void Function(ShoppingTransactionBuilder b) updates,
  ]) = _$ShoppingTransaction;

  ShoppingTransaction._();

  @nullable
  Money get total;
}

/// Searches instances of [ShoppingTransaction].
abstract class ShoppingTransactionQuery extends Object
    with EntityQueryMixin
    implements
        Built<ShoppingTransactionQuery, ShoppingTransactionQueryBuilder> {
  static Serializer<ShoppingTransactionQuery> get serializer =>
      _$shoppingTransactionQuerySerializer;

  factory ShoppingTransactionQuery({
    MetaQuery meta,
    MoneyQuery total,
  }) = _$ShoppingTransactionQuery._;

  factory ShoppingTransactionQuery.build(
          [void Function(ShoppingTransactionQueryBuilder b) updates]) =
      _$ShoppingTransactionQuery;

  ShoppingTransactionQuery._();

  @nullable
  MoneyQuery get total;
}
