// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:dent/account.dart';
import 'package:dent/core.dart';

part 'agreement.g.dart';

/// An agreement between parties.
abstract class Agreement extends Object
    with EntityMixin
    implements Built<Agreement, AgreementBuilder> {
  static Serializer<Agreement> get serializer => _$agreementSerializer;

  factory Agreement({
    Meta meta,
    String typeName,
    Text text,
  }) = _$Agreement._;

  factory Agreement.build([
    void Function(AgreementBuilder b) updates,
  ]) = _$Agreement;

  Agreement._();

  @nullable
  Account get signer;

  @nullable
  String get typeName;

  @nullable
  Text get text;
}

/// Searches instances of [Agreement].
abstract class AgreementQuery extends Object
    with EntityQueryMixin
    implements Built<AgreementQuery, AgreementQueryBuilder> {
  static Serializer<AgreementQuery> get serializer =>
      _$agreementQuerySerializer;

  factory AgreementQuery({
    MetaQuery meta,
    StringQuery typeName,
    TextQuery text,
  }) = _$AgreementQuery._;

  factory AgreementQuery.build([
    void Function(AgreementQueryBuilder b) updates,
  ]) = _$AgreementQuery;

  AgreementQuery._();

  @nullable
  AccountQuery get signer;

  @nullable
  StringQuery get typeName;

  @nullable
  TextQuery get text;
}
