// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:database/database.dart';
import 'package:dent/account.dart';
import 'package:dent/business.dart';
import 'package:dent/core.dart';
import 'package:test/test.dart';

void main() {
  group('Entity', () {
    Database database;
    Document document;
    Product product;

    setUp(() {
      // Set datastore
      database = MemoryDatabaseAdapter().database();
      document = database.collection('product').document('example');

      // Set entity
      product = Product.build((b) {
        b.meta = MetaBuilder()
          ..updatedAt = DateTime.fromMillisecondsSinceEpoch(0, isUtc: true)
          ..owner = (AccountBuilder()
            ..name = 'user name'
            ..phoneNumber = PhoneNumber('000')
            ..emailAddress = EmailAddress(
              'example@gmail.com',
            ));
        b.price = Money.usd(3.14);
        b.description = Text('abc');
      });
    });

    test('datastoreSerializers', () {
      expect(
        product.meta.owner.emailAddress,
        EmailAddress('example@gmail.com'),
      );
      expect(
        datastoreSerializers.serialize(product),
        {
          '@type': 'Product',
          '@meta': {
            'owner': {
              'name': 'user name',
              'emailAddress': 'example@gmail.com',
              'phoneNumber': '000',
            },
            'updatedAt': '1970-01-01T00:00:00.000Z',
          },
          'description': 'abc',
          'price': 'USD 3.14',
        },
      );
    });

    test('datastoreSerializers (with document)', () {
      final example = product.rebuild((b) {
        b.meta ??= MetaBuilder();
        b.meta.document = document;
      });

      expect(
        example.meta.document,
        document,
      );
      final data =
          datastoreSerializers.serialize(example) as Map<String, Object>;
      expect(data, isNotEmpty);
      expect(data['@meta'], isNotNull);
      expect(data['price'], 'USD 3.14');

      final meta = data['@meta'] as Map<String, Object>;
      expect(meta['document'], same(document));
    });
  });
}
