// Copyright 2019 terrier989@gmail.com.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:dent/core.dart';
import 'package:test/test.dart';

void main() {
  group('Text', () {
    test('raw', () {
      expect(Text('').raw, '');
      expect(Text('abc').raw, 'abc');
      expect(Text(':abc').raw, ':abc');
      expect(Text('::abc').raw, ':::abc');
      expect(Text(':::abc').raw, '::::abc');

      expect(Text.fromRaw('').raw, '');
      expect(Text.fromRaw('abc').raw, 'abc');
      expect(Text.fromRaw(':abc').raw, ':abc');
      expect(Text.fromRaw('::abc').raw, '::abc');
      expect(Text.fromRaw(':::abc').raw, ':::abc');
    });

    test('get(): simple', () {
      expect(Text('').get(), '');
      expect(Text('abc').get(), 'abc');
      expect(Text(':abc').get(), ':abc');
      expect(Text('::abc').get(), '::abc');
      expect(Text(':::abc').get(), ':::abc');

      expect(Text.fromRaw('').get(), '');
      expect(Text.fromRaw('abc').get(), 'abc');
      expect(Text.fromRaw(':').get(), ':');
      expect(Text.fromRaw(':abc').get(), ':abc');
      expect(Text.fromRaw('::').get(), '');
      expect(Text.fromRaw('::abc').get(), '');
      expect(Text.fromRaw(':::').get(), '::');
      expect(Text.fromRaw(':::abc').get(), '::abc');
    });

    test('get(): complex', () {
      const source = '''
:: ?en
flip flops
::
:: ?en-AU ?en-NZ // Comment
::
thongs''';
      final text = Text.fromRaw(source);
      expect(
        text.get(localeNames: ['en']),
        'flip flops',
      );
      expect(
        text.get(localeNames: ['en-US']),
        'flip flops',
      );
      expect(
        text.get(localeNames: ['en-AU']),
        'thongs',
      );
      expect(
        text.get(localeNames: ['en-NZ']),
        'thongs',
      );
      expect(
        text.get(localeNames: ['fr']),
        'flip flops',
      );
    });
  });
}
